﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using Windows.ApplicationModel;
using Windows.UI.Xaml.Controls;
using Todolist.Commons;
using Todolist.Helpers;
using Todolist.Models;

namespace Todolist.ViewModels
{
    public class MainPageViewModel : BindableBase
    {
        /// <summary>
        /// 기본 생성자
        /// </summary>
        public MainPageViewModel()
        {
            ViewStatus = new List<PublicModel>
            {
                new PublicModel{Id = "0", Name = "전체", Code = ""},
                new PublicModel{Id = "1", Name = "미완료", Code = StaticDatas.WORKING},
                new PublicModel{Id = "2", Name = "완료", Code = StaticDatas.COMPLETE},
            };

            SelectedView = ViewStatus.FirstOrDefault(p => p.Id == "1");

            Todos = new ObservableCollection<TodoModel>();

            if (DesignMode.DesignModeEnabled)
            {
                Todos = new List<TodoModel>                 
                {
                    new TodoModel{ Id = 0, Title = "todo0", Content = "todotodo", Status = StaticDatas.COMPLETE, Created = DateTime.Now},
                    new TodoModel{ Id = 1, Title = "todo1", Content = "todotodo", Status = StaticDatas.COMPLETE, Created = DateTime.Now},
                    new TodoModel{ Id = 2, Title = "todo2", Content = "todotodo", Status = StaticDatas.WORKING, Created = DateTime.Now},
                    new TodoModel{ Id = 3, Title = "todo3", Content = "todotodo", Status = StaticDatas.WORKING, Created = DateTime.Now},
                    new TodoModel{ Id = 4, Title = "todo4", Content = "todotodo", Status = StaticDatas.COMPLETE, Created = DateTime.Now},
                    new TodoModel{ Id = 5, Title = "todo5", Content = "todotodo", Status = StaticDatas.WORKING, Created = DateTime.Now},
                    new TodoModel{ Id = 6, Title = "todo6", Content = "todotodo", Status = StaticDatas.WORKING, Created = DateTime.Now},
                    new TodoModel{ Id = 7, Title = "todo7", Content = "todotodo", Status = StaticDatas.COMPLETE, Created = DateTime.Now},
                };
            }
            else
            {
                RefreshListAsync();
            }
            CurrentTodo = Todos.FirstOrDefault();
        }

        public IList<PublicModel> ViewStatus { get; set; }

        public ICollection<TodoModel> Todos { get; set; }

        private TodoModel _currentTodo;
        private int _viewSelectedIndex;

        /// <summary>
        /// CurrentTodo
        /// </summary>
        public TodoModel CurrentTodo
        {
            get { return _currentTodo; }
            set { SetProperty(ref _currentTodo, value); }
        }

        private RelayCommand _addCommand;
        private RelayCommand _saveCommand;
        private RelayCommand _completeCommand;
        private RelayCommand _deleteCommand;
        private RelayCommand _comboSelectionChangedCommand;
        private IList<TodoModel> _allTodos;
        private PublicModel _selectedView;

        /// <summary>
        /// 추가 커맨드
        /// </summary>
        public RelayCommand AddCommand
        {
            get 
            { 
                return _addCommand = _addCommand ?? new RelayCommand(async obj =>
                {
                    var maxId = Todos.Max(p => p.Id);
                    maxId++;
                    var newItem = new TodoModel
                    {
                        Id = maxId,
                        Title = "new items",
                        Created = DateTime.Now,
                        Status = StaticDatas.WORKING,
                    };
                    Todos.Add(newItem);
                    await DBHelper.Instance.AddTodoAsync(newItem);
                    CurrentTodo = newItem;
                }); 
            }
        }
        /// <summary>
        /// 저장
        /// </summary>
        public RelayCommand SaveCommand
        {
            get
            {
                return _saveCommand = _saveCommand ?? new RelayCommand(async obj =>
                {
                    if (CurrentTodo == null) return;
                    await DBHelper.Instance.UpdateTodoAsync(CurrentTodo);
                });
            }
        }
        /// <summary>
        /// 완료 커맨드
        /// </summary>
        public RelayCommand CompleteCommand
        {
            get
            {
                return _completeCommand = _completeCommand ?? new RelayCommand(async obj =>
                {
                    if(CurrentTodo == null) return;
                    CurrentTodo.Status = StaticDatas.COMPLETE;
                    CurrentTodo.Updated = DateTime.Now;
                    await DBHelper.Instance.UpdateTodoAsync(CurrentTodo);
                    RefreshListAsync();
                });
            }
        }
        /// <summary>
        /// 삭제 커맨드
        /// </summary>
        public RelayCommand DeleteCommand
        {
            get
            {
                return _deleteCommand = _deleteCommand ?? new RelayCommand(async obj => 
                {
                    if(CurrentTodo == null) return;
                    var deleteItem = CurrentTodo;
                    Todos.Remove(deleteItem);
                    await DBHelper.Instance.DeleteTodoAsync(deleteItem);
                    CurrentTodo = null;
                });
            }
        }

        public RelayCommand ComboSelectionChangedCommand
        {
            get
            {
                return _comboSelectionChangedCommand = _comboSelectionChangedCommand ?? new RelayCommand(obj =>
                {
                    RefreshListAsync();
                });
            }
            
        }

        private async void RefreshListAsync()
        {
            if (SelectedView == null || Todos == null) return;

            Todos.Clear();

            var addItems = await (from kkk in DBHelper.Instance.TodoTable
                where kkk.Status == SelectedView.Code
                      || SelectedView.Code == string.Empty
                orderby kkk.Created descending 
                select kkk).ToListAsync();

            foreach (var item in addItems)
            {
                Todos.Add(item);
            }
            
        }

        /// <summary>
        /// 선택된 뷰
        /// </summary>
        public PublicModel SelectedView
        {
            get { return _selectedView; }
            set { SetProperty(ref _selectedView ,value); }
        }
    }
}
